.hide {
  display: none !important;
}
.image_display {
  height: 100%;
}
.image_preview_holder {
  flex-grow: 1;
  height: calc(100% - 36px);
  display: flex;
  justify-content: center;
  align-items: center;
  background-color: #CCCCCC;
}
.image_preview {
  max-width: 100%;
  max-height: 100%;
}
.hidden_upload {
  display: none;
}
.image_editor_overlay {
  display: flex;
  justify-content: center;
  align-items: center;
}
.image_editor_holder {
  height: 85%;
  width: 85%;
}
.image_editor {
  background-color: black;
}
#tie-btn-reset, #tie-btn-delete, #tie-btn-delete-all {
  display: none !important;
}
.tui-image-editor-icpartition {
  background-color: transparent !important;
}
.tui_close {
  border-radius: 0 !important;
  border: none !important;
  margin-left: 10px !important;
  font-family: 'Open Sans', sans-serif !important;
}
