.table_holder {
  max-width: 100%;
  max-height: 100%;
  overflow: scroll;
  display: none;
}
.csv_preview {
  background-color: white;
  max-width: 100%;
  max-height: 100%;
  font-size: 12px;
  font-family: monospace;
}
.csv_preview tr {
  border-bottom: solid 1px black;
}
.csv_preview tr.header td {
  background-color: #EEA45D;
  font-weight: bold;
}
.csv_preview td {
  padding: 2px 4px;
}
.csv_preview td:nth-child(even) {
  background-color: #EEEEEE;
}
