<div class="gradio output classifier">
  <div class="panel_head">
    <div class="role">Output</div>
    <div class="loading">
      <img src="../static/img/logo_mini.png" class="ld ld-skew"/>
      <span class="loading_time"></span>
    </div>
  </div>
  <div class="output_class"></div>
  <div class="confidence_intervals">
  </div>
</div>
<script src="../static/js/interfaces/output/label.js"></script>
