.panel {
  min-width: 300px;
  flex-grow: 1;
  display: inline-block;
  max-width: 45%;
  min-width: 300px;
  box-sizing: border-box;
  margin: 0 14px 14px;
}
.panel_header, .interface {
  background-color: #F6F6F6;
}
.panel_header {
  text-transform: uppercase;
  font-family: Arial;
  color: #888;
  padding: 6px;
  font-size: 14px;
  font-weight: bold;
  display: flex;
}
.interface {
  height: 360px;
  margin-bottom: 16px;
  padding: 0 6px 6px;
  display: flex;
  flex-flow: column;
}
.loading {
  margin-left: auto;
}
.loading img {
  display: none;
  height: 18px;
}
.panel_buttons {
  display: flex;
}
.submit, .clear, .flag {
  background-color: #F6F6F6 !important;
  flex-grow: 1;
  padding: 8px !important;
  box-sizing: border-box;
  text-transform: uppercase;
  font-weight: bold;
  border: 0 none !important;
}
.submit {
  background-color: #EEA45D !important;
  color: white !important;
}
.submit, .flag_message {
  flex-grow: 2 !important;
  margin-right: 8px;
}
.flag_message {
  padding: 8px !important;
  background-color: #F6F6F6 !important;
}
.clear, .flag {
  margin-left: 8px;
}

.upload_zone {
  font-weight: bold;
  font-size: 24px;
  color: #BBB;
  cursor: pointer;
  width: 100%;
  height: 100%;
  box-sizing: border-box;
  display: flex;
  align-items: center;
  justify-content: center;
  text-align: center;
  line-height: 1.5em;
}
.drop_zone {
  border: dashed 8px #DDD;
}
.edit_holder {
  display: flex;
  justify-content: flex-end;
  margin-bottom: 6px;
}
.interface_button {
  padding: 6px;
  text-transform: uppercase;
  font-weight: bold;
  font-size: 14px;
}
.overlay {
  position: absolute;
  height: 100vh;
  width: 100vw;
  position: fixed;
  z-index: 1;
  background-color: rgba(0, 0, 0, 0.7);
  top: 0;
  left: 0;
}
.flag.flagged {
  background-color: pink !important;
}
