body {
  font-family: 'Open Sans', sans-serif;
  font-size: 12px;
  margin: 0;
}
button, input[type="submit"], input[type="reset"], input[type="text"], input[type="button"], select[type="submit"] {
	background: none;
	color: inherit;
	border: none;
	padding: 0;
	font: inherit;
	cursor: pointer;
	outline: inherit;
  -webkit-appearance: none;
  border-radius: 0;
}
nav, #panels, #share_row {
  margin-left: 60px;
  margin-right: 60px;
}
nav {
  text-align: center;
  padding: 16px 0 4px;
}
nav img {
  margin-right: auto;
  height: 32px;
}
#share_row {
  justify-content: center;
  margin-bottom: 10px;
}
#share_form {
  flex-grow: 1;
  justify-content: center;
  align-items: center;
}
#share_row button, #share_row input[type=text] {
  padding: 6px 4px;
  margin-left: 5px;
}
#share_row input[type=text] {
  background-color: #F6F6F6;
}
#share_email {
  flex-grow: 1;
  max-width: 400px;
}
#share_row, #share_complete, #share_form {
  display: none;
}
#panels {
  display: flex;
  flex-flow: row;
  flex-wrap: wrap;
  justify-content: center;
}
button.primary {
  color: white;
  background-color: #EEA45D;
}
button.secondary {
  color: black;
  background-color: #F6F6F6;
}
