<div class="gradio input image_file">
  <div class="role">Input</div>
  <div class="input_image drop_mode">
    <div class="input_caption">Drop Image Here<br>- or -<br>Click to Upload</div>
    <img />
  </div>
  <input class="hidden_upload" type="file" accept="image/x-png,image/gif,image/jpeg" />
</div>

<link rel="stylesheet" href="../static/css/vendor/cropper.css">
<script src="../static/js/vendor/cropper.js"></script>
<script src="../static/js/interfaces/input/image_upload.js"></script>
