<div class="gradio input csv">
  <div class="role">Input</div>
  <div class="input_csv drop_mode">
    <div class="input_caption">Drop CSV File Here<br>- or -<br>Click to Upload</div>
  </div>
  <div class="table_holder"><table class="csv_preview"></table></div>
  <input class="hidden_upload" type="file" accept=".csv" />
</div>

<script src="../static/js/vendor/papaparse.min.js"></script>
<script src="../static/js/interfaces/input/csv.js"></script>
